import { observable, action } from 'mobx'
import { NetStore, UserStore } from '../store'
import { CzbApi, CzbCourseType } from '../model'
import { Store } from '../Store'
import { Platform} from 'react-native'
class Classroom {

    AgoraAppID = '17fe5078e7c9467980c1bccfea996fe3'
    CCAccountID = 'E690524F502FB210'

    @observable liveStatus = 0
    @observable teacherName
    @observable micStatus = false//only for cc
    @observable raiseHand = false//only for cc
    @observable isTeacherEnterClassRoom = false//老师是否进入教室
    @observable argoraOtmIsShow = false//声网一对多远程视频是否渲染
    @observable argoraOtmLocalIsShow = false //声网一对多本地视频是否显示
    @observable argoraIsShow = false //声网一对一远程视频是否渲染
    @observable argoraLocalIsShow = false //声网一对一本地视频是否显示
    /**
     * 验证教室
     * @param roomId 教室ID
     */
    authCourse = (roomId) => {
        return new Promise((resolve, reject) => {
            NetStore.fetch(CzbApi.kAuthClassroom, null, {  roomId: roomId })
                .then(response => {
                    console.log('authCourse',response);
                    const {code,data} = response;
                    if(code == 200){
                        resolve(response)
                    }else{
                        throw response;
                    }
                })
                .catch(error => {
                    console.log('authCourse','error',error);
                    reject(error)
                })
        })
    }

    /**
     * 获取agora channel key
     * courseID 课程id
     *
     * return promise
     * @memberof ClassroomStore
     */
    fetchChannelKey = (courseID) => {
        return new Promise((resolve, reject) => {
            NetStore.fetch(CzbApi.kFetchChannelKeyAndroid,{ bookedId: courseID, userType: 1, userId: UserStore.id})
                .then(response => {
                    const { result } = response
                    resolve(result)
                })
                .catch(error => {
                    reject(error)
                })
        })
    };

    /**
     * 获取Agora Token
     * @param roomId  教室id
     * @memberof Classroom
     */
    fetchAgoraToken = (roomId) => {
        return new Promise((resolve, reject) => {
            NetStore.fetch(CzbApi.GetAgoraChannelToken, { roomId: roomId, userType: 1, userId: UserStore.id })
                .then(response => {
                    const {code,data} = response;
                    console.log('fetchAgoraToken',response)
                    if(code == 200){
                        resolve(data)
                    }else{
                        throw response
                    }
                })
                .catch(error => {
                    console.log('fetchAgoraToken','error',error)
                    reject(error)
                })
        })
    }

    /**
     * 获取CC Session ID
     * roomID cc房间号
     * password cc房间密码
     *
     * return Promise
     * @memberof ClassroomStore
     */
    fetchCCSession = (roomID, password) => {
        let url = `${CzbApi.kFetchCCSession}/${UserStore.id}`
        let data = {
            client: 1,
            role: 1,
            name: UserStore.name,
            password,
            roomid: roomID,
            userid: this.CCAccountID,
        }
        return new Promise((resolve, reject) => {
            NetStore.fetch(url, data)
                .then(response => {
                    const { statusCode, sessionid } = response
                    if (statusCode == 200) {
                        resolve(sessionid)
                    }
                    else {
                        reject({ errorCode: statusCode, errorMsg: '获取session失败' })
                    }
                })
                .catch(error => {
                    reject(error)
                })
        })
    }

    /**
     * 上传Agora错误日志
     * roomId: 房间id
     * channelKey: agora channel key
     * method: 发生错误的方法
     * message: 错误信息
     * return Promise
     * @memberof Classroom
     */
    uploadAgoraError = (roomId, channelKey = '', method, message) => {
        return new Promise((resolve, reject) => {
            let data = {
                roomId: roomId,
                roleType:1,
                agoraToken: channelKey ? channelKey : '',
                clientDetails: `device=${Store.deviceInfo}&app=${Store.appVersion}`,
                errorMsg: message,
                otherThirdType: 3,
                errorMethod: method,
            }
            NetStore.fetch(CzbApi.kUploadLog, data)
                .then(response => {
                    resolve(response)
                })
                .catch(error => {
                    reject(error)
                })
        })
    }

    /**
     * 上传CC错误日志
     * classroomID: cc房间号
     * sessionID： cc session id
     * method：发生错误的方法
     * message：错误信息
     *
     * return Promise
     * @memberof Classroom
     */
    uploadCCError = (classroomID, sessionID, method, message) => {
        return new Promise((resolve, reject) => {
            let data = {
                roomid: classroomID,
                sessionid: sessionID ? sessionID : '',
                clientDetails: `device=${Store.deviceInfo}&app=${Store.appVersion}`,
                errorMsg: message,
                otherThirdType: 2,
                errorMethod: method,
            }
            NetStore.fetch(CzbApi.kUploadLog, data)
                .then(response => {
                    resolve(response)
                })
                .catch(error => {
                    reject(error)
                })
        })
    }

    /**
     * 设置声网一对多是否运端流加载完成要显示
     * @param isShow 是不是要渲染远端流
     * */
    @action
    setAgoraOtmShow(isShow){
        this.argoraOtmIsShow = isShow
    }

    /**
     * 设置声网一对多本地视频是否显示
     * @param isShow 是不是要渲染本地流
     * */
    setAgoraOtmLocalShow(isShow){
        this.argoraOtmLocalIsShow = isShow
    }

    /**
     * 设置声网一对一是否运端流加载完成要显示
     * @param isShow 是不是要渲染远端流
     * */
    @action
    setAgoraShow(isShow){
        this.argoraIsShow = isShow
    }

    /**
     * 设置声网一对一本地视频是否显示
     * @param isShow 是不是要渲染本地流
     * */
    setAgoraLocalShow(isShow){
        this.argoraLocalIsShow = isShow
    }

    /**
     * 重置ClassroomStore
     *
     * @memberof ClassroomStore
     */
    @action
    clear = () => {
        this.liveStatus = 0
        this.teacherName = null
        this.micStatus = false
        this.raiseHand = false
        this.argoraOtmIsShow = false
        this.argoraOtmLocalIsShow = false
        this.argoraIsShow = false
        this.argoraLocalIsShow = false
    }

    /**
     * @desc 用于设置老师是否进入教室的状态
     * @param isEnter表示老师是否进入教室
     * @return void
     * */
    @action
    setTeacherEnterClassroomState(isEnter){
        this.isTeacherEnterClassRoom = isEnter
    }
}

export let ClassroomStore = new Classroom
