import { observable, action, computed, runInAction } from 'mobx'
import { CzbApi, CzbCourseType, CzbError, TimeCellModel, ClassroomModel, GradeModel } from '../model'
import { NetStore, UserStore } from '../store'
import { Store } from '../Store'
import moment from 'moment'

class Reservation {

    /**
     * 预约模式(1v1正式课、小班正式课、小班体验课)
     *
     * @memberof ReservationStore
     */
    @observable selectedCourseType = CzbCourseType.oto

    /* 预约的时间 */
    /**
     * 可预约时间数据
     *
     * @memberof ReservationStore
     */
    @observable timeModels = []

    /**
     * 已选的预约时间model
     *
     * @memberof ReservationStore
     */
    @observable selectedTimeModel = null

    /**
     * 已选时间的字符串
     *
     * @readonly
     * @memberof ReservationStore
     */
    @computed
    get selectedTimeString() {
        if (this.selectedTimeModel) {
            return this.selectedTimeModel.timeTag
        }
        return null
    }

    /* 预约学年 */
    /**
     * 学年原始数据
     *
     * @memberof ReservationStore
     */
    @observable gradeModels = [
        new GradeModel('KGL2', '幼儿园中班', this),
        new GradeModel('KGL3', '幼儿园大班', this),
        new GradeModel('PSL1', '小学一年级', this),
        new GradeModel('PSL2', '小学二年级', this),
        new GradeModel('PSL3', '小学三年级', this),
    ]

    /**
     * 已选的学年
     *
     * @memberof ReservationStore
     */
    @observable selectedGrade = null

    /**
     * 已选学年的名字
     *
     * @readonly
     * @memberof ReservationStore
     */
    @computed
    get selectedGradeName() {
        if (this.selectedGrade) {
            return this.selectedGrade.name
        }
        return '未选择'
    }

    /**
     * 已选学年的id
     *
     * @readonly
     * @memberof ReservationStore
     */
    @computed
    get selectedGradeID() {
        if (this.selectedGrade) {
            return this.selectedGrade.id
        }
        return null
    }


    /**预约1v1 */
    /**
     * 是否有1v1课程
     *
     * @memberof ReservationStore
     */
    @observable isHaveOtoCourse = true
    /**
     * 是否有可预约的时间
     *
     * @memberof ReservationStore
     */
    @observable isEnabledTime = false
    /**
     * 课程一级领域名称
     *
     * @memberof ReservationStore
     */
    @observable courseFirstName = '未选择'

    /**
     * 课程二级领域名称
     *
     * @memberof ReservationStore
     */
    @observable courseSecondName = '未选择'

    /**
     * 预约使用的课时数量
     *
     * @memberof ReservationStore
     */
    @observable shouldCostHour = 0

    /**
     * 获取1v1可预约时间,接口返回是否有课、课程名称、消耗课时数量
     * date 预约日期 'YYYY-MM-DD'
     * return promise
     *
     * @memberof ReservationStore
     */
    fetchOtoEnabledTime = (date) => {
        let orderType = this.selectedCourseType == CzbCourseType.oto ? 1 : 2
        let userID = UserStore.id
        let year = moment(date).get('year')
        let month = moment(date).get('month') + 1
        let day = moment(date).get('date')
        let url = `${CzbApi.kFetchCourseTime}/${userID}/${orderType}/${year}-${this.prefixInteger(month, 2)}-${this.prefixInteger(day, 2)}`

        return new Promise((resolve, reject) => {
            NetStore.fetch(url, null)
                .then(response => {
                    console.log('1v1可预约时间 ==>')
                    console.log(response)
                    if (!response.courseTag) {
                        this.isHaveOtoCourse = false
                        reject({ errorCode: 8000, errorMsg: CzbError["8000"] })
                    }
                    else {
                        this.isHaveOtoCourse = true
                        this.parseOtoEnabledTime(response)
                        resolve()
                    }
                })
                .catch(error => {
                    reject(error)
                })
        })
    }

    /**
     * 解析1v1可预约时间
     * 设置timeModels，courseFirstName，courseSecondName，shouldCostHour
     *
     * @memberof ReservationStore
     */
    parseOtoEnabledTime = (data) => {
        const {
            fieldOneName: first,
            fieldTwoName: second,
            currBookTimeNums: times,
            useRecord: shouldCostHour,
        } = data
        this.timeModels = times.map((value, index) => {
            return new TimeCellModel(index, value, this)
        })
        this.courseFirstName = first
        this.courseSecondName = second
        this.shouldCostHour = shouldCostHour
        this.isEnabledTime = this.isHaveEnabledTime(this.timeModels.slice())
    }

    /**
     * 判断时间数组是否有可预约时间
     * return bool
     * @memberof ReservationStore
     */
    isHaveEnabledTime = (array) => {
        return array.some(value => value.number > 0)
    }

    //补位0
    prefixInteger = (num, n) => {
        return (Array(n).join(0) + num).slice(-n)
    }

    /**
     * 预约1v1课程
     * date: 'YYYY-MM-DD'
     * time: 'HH:mm'
     *
     * @memberof ReservationStore
     */
    orderOtoCourse = (date, time) => {
        let userID = UserStore.id
        var courseInfo
        if (this.selectedCourseType === CzbCourseType.oto) {
            courseInfo = `"${this.courseFirstName}"-"${this.courseSecondName}"`
        }
        else if (this.selectedCourseType === CzbCourseType.otoExp) {
            courseInfo = '体验课'
        }
        let successTip = `您已预约 ${date} ${time}\n${courseInfo} 课程`
        let hour = time.slice(0, 2)
        let minute = time.slice(3, 5)
        let orderType = this.selectedCourseType == CzbCourseType.oto ? 1 : 2
        let url = `${CzbApi.kCourseReservation}/${userID}/${orderType}/${Store.courseID}/${date}/${hour}/${minute}`
        return new Promise((resolve, reject) => {
            NetStore.fetch(url, null)
                .then(response => {
                    const { resInfo: msg, handelRes: result } = response
                    if (!result) {
                        reject({ errorCode: 8001, errorMsg: CzbError['8001'] })
                        return
                    }
                    if (msg === 'success') {
                        resolve(successTip)
                    }
                    else if (msg === 'fail') {
                        reject({ errorCode: 8002, errorMsg: CzbError['8002'] })
                    }
                    else if (msg === 'fail1') {
                        reject({ errorCode: 8003, errorMsg: CzbError['8003'] })
                    }
                    else if (msg === 'fail2') {
                        reject({ errorCode: 8004, errorMsg: CzbError['8004'] })
                    }
                    else if (msg === 'fail3') {
                        reject({ errorCode: 8005, errorMsg: CzbError['8005'] })
                    }
                })
                .catch(error => {
                    reject(error)
                })
        })
    }

    /**
     * 取消1v1课程
     * courseID 课程id
     * return promise
     * @memberof ReservationStore
     */
    cancelOtoCourse = (courseID) => {
        return new Promise((resolve, reject) => {
            NetStore.fetch(CzbApi.kCancelReservation, { bookedId: courseID } )
                .then(response => {
                    resolve()
                })
                .catch(error => {
                    reject(error)
                })
        })
    }

    /** 小班课 */

    /**
     * 可选教室数据
     *
     * @memberof ReservationStore
     */
    @observable classroomModels = null

    /**
     * 已选的教室
     *
     * @memberof ReservationStore
     */
    @observable selectedClassroom = null

    /**
     * 已选教室的名字
     *
     * @readonly
     * @memberof ReservationStore
     */

    @computed
    get selectedClassroomName() {
        if (this.selectedClassroom) {
            return this.selectedClassroom.name
        }
        return '未选择'
    }

    /**
     * 已选教室的id
     *
     * @readonly
     * @memberof ReservationStore
     */
    @computed
    get selectedClassroomID() {
        if (this.selectedClassroom) {
            return this.selectedClassroom.id
        }
        return null
    }

    /**
     * 设置小班课时间，只有19点和20点
     * model中number不能传0，否则不显示cell
     *
     * @memberof ReservationStore
     */
    setOtmTimeModels = () => {
        this.selectedTimeModel = null
        this.timeModels = [
            new TimeCellModel(38, 1, this),//19:00
            new TimeCellModel(40, 1, this),//20:00
        ]
    }

    /**
     * 获取小班课（正式课和体验课）可预约班级
     *
     * return promise
     * @memberof ReservationStore
     */
    fetchOtmClassroom = () => {
        this.selectedClassroom = null
        if (this.selectedCourseType == CzbCourseType.otm) {
            this.shouldCostHour = 2.5
        }
        else if (this.selectedCourseType == CzbCourseType.otmExp) {
            this.shouldCostHour = 1
        }
        return new Promise((resolve, reject) => {
            NetStore.fetch(CzbApi.kFetchMultiCourseClass, null)
                .then(response => {
                    this.classroomModels = []
                    if (response && response.length > 0) {
                        this.classroomModels = response.map((value) => {
                            return new ClassroomModel(value.id, value.name, this)
                        })
                        resolve()
                    }
                })
                .catch(error => {
                    reject(error)
                })
        })
    }

    /**
     * 预约小班课
     * gradeID 学年id
     * classroomID 教室id
     * date 上课日期
     * time 上课时间
     *
     * return promise
     * @memberof ReservationStore
     */
    orderOtmCourse = (gradeID, classroomID, date, time) => {
        let timestring = `${date} ${time}:00`
        let timestamp = moment(timestring).valueOf()
        let classType = this.selectedCourseType === CzbCourseType.otmExp ? 4 : 6//4：1vN体验课 6：1vN正式课
        let successTip = `您已预约 ${date} ${time}\n"${this.selectedGradeName}-"${this.selectedClassroomName}"课程`
        let reservationURL
        if (this.selectedCourseType == CzbCourseType.otm) {
            reservationURL = `${CzbApi.kOrderOtm}/${gradeID}/${classroomID}/${timestamp}`
        }
        else if (this.selectedCourseType == CzbCourseType.otmExp) {
            reservationURL = `${CzbApi.kOrderOtmExp}/${gradeID}/${classroomID}/${timestamp}`
        }
        return new Promise((resolve, reject) => {
            NetStore.fetch(reservationURL, null)
                .then(response => {
                    resolve(successTip)
                })
                .catch(error => {
                    reject(error)
                })
        })
    }

    /**
     * 取消小班课
     * courseID 课程id
     * courseType 课程类型 正式课或体验课
     * queueID 如果是体验课，传队列id
     *
     * return promise
     *
     * @memberof ReservationStore
     */
    cancelOtmCourse = (courseID, courseType, queueID) => {
        let userID = UserStore.id
        let url
        if (courseType == CzbCourseType.otm) {
            return new Promise((resolve, reject) => {
                NetStore.fetch(CzbApi.kCancelMultiCourse, { userId: userID, classId: courseID, cType: CzbCourseType.otm } )
                    .then(response => {
                        resolve()
                    })
                    .catch((error) => {
                        reject(error)
                    })
            })

        }
        else if (courseType == CzbCourseType.otmExp) {
            return new Promise((resolve, reject) => {
                if (!queueID && courseID) {
                    reject({ errorCode: 8012, errorMsg: CzbError["8012"] })
                    return
                }
                NetStore.fetch(CzbApi.kCancelOtmExp, null, { queueId: queueID })
                    .then(response => {
                        resolve('取消课程成功')
                    })
                    .catch(error => {
                        reject(error)
                    })
            })
        }
    }

    /**
     * @desc 取消课程
     * @param roomId 教室ID
     * */
    cancelCourse = (roomId)=>{
        return new Promise((resolve, reject) => {
            NetStore.fetch(CzbApi.kCancelCourse, null, { roomId: roomId })
                .then(response => {
                    console.log('cancelCourse',response)
                    const {code} = response;
                    if(code == 200){
                        resolve('取消课程成功')
                    }else{
                        throw response
                    }
                })
                .catch(error => {
                    console.log('cancelCourse','error',error)
                    reject(error)
                })
        })
    }
}

export let ReservationStore = new Reservation
